<video id="{{ video.id }}" class="video-js vjs-default-skin" controls preload="auto" {% if width %}width="{{ width }}" height="{{ height }}"{% endif %} {% if video.poster_url %}poster="{{ video.poster_url }}"{% endif %} data-setup="" >
    {% for source in video.sources %}
        <source src="{{ source.src }}" type="{{ source.type }}">
    {% endfor %}
</video>